﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.Web.UI.DataVisualization.Charting;

namespace DrillDownChart
{
    public partial class RegionReport : System.Web.UI.Page
    {
        // Step 1 - Create Example Data
        SalesDataSet ExampleData = new SalesDataSet();

        protected void Page_Load(object sender, EventArgs e)
        {
            // Step 2 - Populate chart drop down
            PopulateChartTypeDropDown();

            // Step 3 - Create Series
            Series series = CreateSeries();

            // Step 4 - Set the chart type
            SetChartType(series);

            // Step 5 - Add data (and if needed drilldown links to series)
            AddDataToSeries(series);

            // Step 6 - Add series into the chart's series collection
            SalesReport.Series.Add(series);
        }

        private void PopulateChartTypeDropDown()
        {
            List<String> chartTypes = new List<String>(Enum.GetNames(typeof(SeriesChartType)));
            chartTypes.Insert(0, "");

            foreach (var item in chartTypes)
            {
                DropDownListChartType.Items.Add(item);
            }
        }

        private Series CreateSeries()
        {
            Series series = new Series("Sales");
            series.BorderWidth = 3;
            series.ShadowOffset = 2;
            return series;
        }

        private void SetChartType(Series inSeries)
        {
            if (Page.Request["ChartType"] != null)
                DropDownListChartType.SelectedValue = Page.Request["ChartType"];

            if (DropDownListChartType.SelectedValue.ToString() == "")
                DropDownListChartType.SelectedValue = SeriesChartType.Column.ToString();

            inSeries.ChartType = (SeriesChartType)System.Enum.Parse(typeof(SeriesChartType), DropDownListChartType.SelectedValue.ToString());
        }

        private void AddDataToSeries(Series series)
        {
            if (Page.Request["ChartType"] == null)
                AddAllRegionData(series);
            else
                AddSpecificRegionData(series);
        }

        private void AddAllRegionData(Series series)
        {
            DataTable sales = ExampleData.Tables["RepsSales"];
            DataTable regions = ExampleData.Tables["Region"];

            var query = from reps in sales.AsEnumerable()
                        join region in regions.AsEnumerable()
                        on reps.Field<int>("RegionID") equals region.Field<int>("RegionID")
                        group reps by region.Field<string>("RegionName") into regionGroup
                        select new { Region = regionGroup.Key, Sales = regionGroup.Sum(total => total.Field<int>("Sales")) };

            // Populate new series with data
            foreach (var value in query)
            {
                series.Points.AddXY(value.Region, value.Sales);
            }

            // Step 7 - Make this series drillable
            for (int i = 0; i < series.Points.Count; i++)
            {
                series.Points[i].Url = string.Format("RegionReport.aspx?region={0}&ChartType={1}", series.Points[i].AxisLabel, DropDownListChartType.SelectedValue);
            }
        }

        private void AddSpecificRegionData(Series series)
        {
            var query = from reps in ExampleData.RepsSalesTable.AsEnumerable()
                        join region in ExampleData.RegionTable.AsEnumerable()
                        on reps.Field<int>("RegionID") equals region.Field<int>("RegionID")
                        where region.Field<string>("RegionName") == (Page.Request["Region"] ?? "East")
                        select new { RepName = reps.Field<string>("Name"), Sales = reps.Field<int>("Sales") };

            // Populate new series with data
            foreach (var value in query)
            {
                series.Points.AddXY(value.RepName, value.Sales);
            }

            // Step 7 - Make this series drillable
            //for (int i = 0; i < series.Points.Count; i++)
            //{
            //    // Add drill down code to drill to a third chart
            //}
        }

        private void AddDrillDown(Series series)
        {
            for (int i = 0; i < series.Points.Count; i++)
            {
                series.Points[i].Url = string.Format("RegionReport.aspx?region={0}&ChartType={1}", series.Points[i].AxisLabel, DropDownListChartType.SelectedValue);
            }
        }
    }
}